<?php
namespace FiloBlu\Checkproducts\Cron;
use \Psr\Log\LoggerInterface;

class Productdatacheck {
    protected $_logger;
    protected $_productRepository;
    protected $_searchCriteriaBuilder;
    protected $_scopeConfig;
    protected $_storeRepository;
    protected $_objectManager;
    protected $_linkManagement;
    protected $_stockRegistry;

    public function __construct(
        LoggerInterface $logger,
        \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Store\Model\StoreRepository $storeRepository,
        \Magento\Framework\ObjectManagerInterface $objectmanager,
        \Magento\ConfigurableProduct\Api\LinkManagementInterface $linkManagement,
        \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry
    ) {
        $this->_logger = $logger;
        $this->_productRepository = $productRepository;
        $this->_searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->_scopeConfig = $scopeConfig;
        $this->_storeRepository = $storeRepository;
        $this->_objectManager = $objectmanager;
        $this->_linkManagement = $linkManagement;
        $this->_stockRegistry = $stockRegistry;
    }

    /**
     * Write to system.log
     *
     * @return void
     */

    public function execute() {

        $active = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_general/checkproducts_enabled');
        if(!$active){
            return;
        }

        $searchCriteria = $this->_searchCriteriaBuilder->addFilter('to_check', '1', 'eq')->create();
        $searchResults = $this->_productRepository->getList($searchCriteria);
        $products = $searchResults->getItems();
        $stores = $this->_storeRepository->getList();
        $problems = [];
        $enabled = [];
        $putProductInStock = false;
        foreach ($products as $product){
            if(isset($product['manual_status']) && intval($product['manual_status'])){
                continue;
            }
            $check = [];
            $type = $product['type_id'];
            $attributes = explode(',', $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_' . $type . '/checkproducts_' . $type . '_attribute', \Magento\Store\Model\ScopeInterface::SCOPE_STORE));
            foreach ($stores as $store) {
                $product = $this->_productRepository->get($product['sku'], true, $store["store_id"], true);

                if(is_array($attributes) && count($attributes) > 0){
                    foreach ($attributes as $attribute){
                       if(!isset($product[$attribute]) || (isset($product[$attribute]) && $product[$attribute] == '')){
                           $check[$attribute][$store["code"]] = false;
                       }
                    }
                }

                $priceCheck = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_' . $type . '/checkproducts_' . $type . '_price', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

                if($priceCheck && $type != 'configurable'){
                    if(!isset($product['price']) || (isset($product['price']) && floatval($product['price']) < 1)){
                        $check['price'][$store["code"]] = false;
                    }
                }
                elseif($priceCheck && $type == 'configurable' && intval($store["store_id"]) == 0){
                    $res = $this->checkConfigurablePrice($product);
                    if(!$res){
                        $check['price_configurable'][$store["code"]] = false;
                    }
                }

                if(intval($store["store_id"]) == 0) {
                    $imageCheck = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_' . $type . '/checkproducts_' . $type . '_image', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

                    if ($imageCheck) {
                        if(!$product->getImage() || trim($product->getImage()) == 'no_selection'){
                            $check['images'][$store["code"]] = false;
                        }
                        /*if (!isset($product['media_gallery']) || (isset($product['media_gallery']) && count($product['media_gallery']) <= 0)) {
                            if (!isset($product['media_gallery']['images']) || (isset($product['media_gallery']['images']) && count($product['media_gallery']['images']) <= 0)) {
                                $check['images'][$store["code"]] = false;
                            }
                        }*/
                    }

                    $categoryCheck = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_' . $type . '/checkproducts_' . $type . '_categories', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

                    if($categoryCheck){
                        if(!isset($product['category_ids']) || (isset($product['category_ids']) && count($product['category_ids']) <= 0)){
                            $check['category'][$store["code"]] = false;
                        }
                    }

                    $qtyCheck = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_' . $type . '/checkproducts_' . $type . '_qty', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

                    $putProductInStock = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_' . $type . '/checkproducts_' . $type . '_put_in_stock', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

                    if($qtyCheck){
                        if (isset($product['quantity_and_stock_status'])) {
                            if ((!isset($product['quantity_and_stock_status']['is_in_stock']) || (isset($product['quantity_and_stock_status']['is_in_stock']) && !$product['quantity_and_stock_status']['is_in_stock'])) && !$putProductInStock) {
                                $check['stock_status'][$store["code"]] = false;
                            }
                            if($type != 'configurable' && (!isset($product['quantity_and_stock_status']['qty']) || (isset($product['quantity_and_stock_status']['qty']) && intval($product['quantity_and_stock_status']['qty']) <= 0))){
                                $check['qty'][$store["code"]] = false;
                            }
                        }
                        else{
                            $check['stock'][$store["code"]] = false;
                        }
                    }
                }
            }
            if(count($check) <= 0) {
                $attributesData['to_check'] = 0;
                $attributesData['status'] = 1;
                try {
                    $this->_objectManager->get('Magento\Catalog\Model\Product\Action')->updateAttributes([$product['entity_id']], $attributesData, 0);
                    //$this->_productRepository->save($product);

                    if ($putProductInStock) {
                        $stockItem = $this->_stockRegistry->getStockItem($product['entity_id']);
                        $stockItem->setData('is_in_stock', 1);
                        $stockItem->save();
                    }
                }
                catch (\Exception $e){
                    $this->_logger->info('Product: '.$product['sku'] . " Error: " . $e->getMessage());
                }
                $enabled[] = $product['sku'];
            }
            else{
                $problems[$product['sku']] = $check;
            }
        }

        $this->sendReport($enabled, $problems);

        $this->_logger->info('End Product Check');
    }

    public function sendReport($enabled, $problems){
        /* Here we prepare data for our email  */

        $sender = $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_email/checkproducts_email_sender', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        if(!isset($sender['name']) || !isset($sender['email']))
        {
            $sender = "general";
        }
        $senderEmail = $this->_scopeConfig->getValue('trans_email/ident_' . $sender . '/email', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $senderName = $this->_scopeConfig->getValue('trans_email/ident_' . $sender . '/name', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

        $receiversEmail = explode(',', $this->_scopeConfig->getValue('filoblu_checkproducts/checkproducts_email/checkproducts_email_receiver', \Magento\Store\Model\ScopeInterface::SCOPE_STORE));


        /* Receiver Detail  */
//        $receiverInfo = [
//            'name' => '',
//            'email' => 'luigi+test@filoblu.com'
//        ];


        /* Sender Detail  */
        $senderInfo = [
            'name' => $senderName,
            'email' => $senderEmail,
        ];


        /* Assign values for your template variables  */
        $emailTemplateVariables = [];
        $emailTempVariables['cron'] = 'Products Data Check';

        $tableBodyProblems = '';
        foreach($problems as $key => $values){

            foreach ($values as $value => $stores ){
                $tableBodyProblems .= '<tr>';
                $tableBodyProblems .= '<td>'. $key .'</td>';
                if($value == 'price') {
                    $tableBodyProblems .= '<td>Price not found</td>';
                }
                elseif ($value == 'images'){
                    $tableBodyProblems .= '<td>Image not found</td>';
                }
                elseif ($value == 'category'){
                    $tableBodyProblems .= '<td>Without category</td>';
                }
                elseif ($value == 'qty'){
                    $tableBodyProblems .= '<td>Qty zero</td>';
                }
                elseif ($value == 'stock_status'){
                    $tableBodyProblems .= '<td>Out of stock</td>';
                }
                elseif ($value == 'stock'){
                    $tableBodyProblems .= '<td>Stock problem</td>';
                }
                elseif ($value == 'price_configurable'){
                    $tableBodyProblems .= '<td>All children have zero price</td>';
                }
                else{
                    $tableBodyProblems .= '<td>Attribute: ' . $value . ' is empty</td>';
                }
                $tableBodyProblems .= '<td>'. implode(", ",array_keys($stores)) .'</td>';
                $tableBodyProblems .= '</tr>';
            }

        }


        $emailTempVariables['tBodyProblems'] = $tableBodyProblems;


        $tableBodyEnabled = '';
        foreach($enabled as $sku){

            $tableBodyEnabled .= '<tr>';
            $tableBodyEnabled .= '<td>'. $sku .'</td>';
            $tableBodyEnabled .= '</tr>';

        }

        $emailTempVariables['tBodyEnabled'] = $tableBodyEnabled;

        /* call send mail method from helper or where you define it*/
        $this->_objectManager->get('FiloBlu\Checkproducts\Helper\Email')->sendCustomEmail(
            $emailTempVariables,
            $senderInfo,
            $receiversEmail
        );
    }

    public function checkConfigurablePrice($configurableProduct){
        $childProducts = $this->_linkManagement->getChildren($configurableProduct['sku']);
        foreach ($childProducts as $children){
            $childrenPrice = true;
            $stores = $this->_storeRepository->getList();
            foreach ($stores as $store) {
                $product = $this->_productRepository->get($children['sku'], true, $store["store_id"], true);
                if(!isset($product['price']) || (isset($product['price']) && floatval($product['price']) < 1)){
                    $childrenPrice = false;
                    Break;
                }
            }
            if($childrenPrice == true){
                return true;
            }
        }
        return false;
    }

}